/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package x33eja.sb;

import java.security.Principal;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.annotation.security.RolesAllowed;
import javax.ejb.SessionContext;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import x33eja.model.Course;
import x33eja.model.Person;
import x33eja.model.Student;
import x33eja.model.Teacher;

/**
 *
 * @author blcha
 */
@Stateless
public class SchoolCenter implements SchoolCenterLocal {

    @PersistenceContext
    EntityManager em;
    
    @Resource
    SessionContext ctx;

    @Override
    public List<Course> getNotEnrolledCoursesForCurrent(Student student) {
        return (List<Course>) em.createQuery("SELECT a FROM Course a WHERE a.courseId NOT IN (SELECT c.courseId FROM Student p JOIN p.mEnrolledIn c WHERE p=:student)").setParameter("student", student).getResultList();

    }

    @RolesAllowed({"teacher"})
    @Override
    public void updateStudent(Student student) {
        em.merge(student);
    }
    
    @RolesAllowed({"teacher"})
    @Override
    public void removeStudent(Student student) {
        student = em.merge(student);
        em.remove(student);
    }

    @RolesAllowed({"teacher", "student"})
    @Override
    public Student getStudent(int birthNumber) {
        Student student = em.find(Student.class, birthNumber);
        return student;
    }

    @Override
    public Course getCourse(String courseId) {
        Course course = em.find(Course.class, courseId);
        return course;
    }

    @Override
    public Map<Student, Long> getStudentsWithCourseCount() {
        final Map<Student, Long> map = new LinkedHashMap<Student, Long>();

        final List<Object[]> result = (List<Object[]>) em.createQuery("SELECT p, count(e) FROM Student p LEFT JOIN p.mEnrolledIn e GROUP BY p ORDER BY p.surname, p.firstName").getResultList();

        for (final Object[] o : result) {
            map.put((Student) o[0], (Long) o[1]);
        }

        return map;
    }

    @Override
    public Teacher getTeacher(int birthNumber) {
        Teacher teacher = em.find(Teacher.class, birthNumber);
        return teacher;
    }

    @Override
    public List<Course> getAllCourses() {
        return (List<Course>) em.createNamedQuery(Course.Q_GET_ALL_COURSES).getResultList();
    }

    @Override
    public List<Student> getAllStudents() {
        return (List<Student>) em.createQuery("SELECT s FROM Student s").getResultList();
    }

    @Override
    public List<Teacher> getAllTeachers() {
        return (List<Teacher>) em.createQuery("SELECT t FROM Teacher t").getResultList();
    }
    
    @Override
    public Person getLoggedUser() {
        Principal p = ctx.getCallerPrincipal();
        if (p == null) {return null;}
        try {
            Person person = (Person) em.createQuery("SELECT p FROM Person p WHERE p.login=:login").setParameter("login", p.getName()).getSingleResult();
            return person;
        } catch (NoResultException nrex) {
            return null;
        }
    }
    
}
